@@ -1809,7 +1809,7 @@ func (m *Manager) switchMode(mode byte, name string) (err error) {
 
 	monitorsId := monitors.getMonitorsId()
 	options := getSwitchModeOptions(mode, name)
-	err = m.switchModeAux(mode, oldMode, monitorsId, monitorMap, true, options)
+	err = m.switchModeAux(mode, oldMode, monitorsId, monitorMap, false, options)
 	if err != nil {
 		applyErr, ok := err.(*applyFailed)
 		if ok && applyErr.reason == reasonNumChanged {
@@ -2466,6 +2466,10 @@ func (m *Manager) saveSysConfigNoLock(reason string) error {
 		return nil
 	}
 
+	if !m.sessionActive {
+		return nil
+	}
+
 	m.sysConfig.UpdateAt = time.Now().Format(time.RFC3339Nano)
 	m.sysConfig.Version = sysConfigVersion
 
